@@ -1,14 +1,17 @@
 package perfSONAR_PS::Messages;
 
 use strict;
+use warnings;
 use Exporter;
 use Log::Log4perl qw(get_logger :nowarn);
 
-our $VERSION = 0.06;
 use perfSONAR_PS::Common;
+use Params::Validate qw(:all);
 
+our $VERSION = 0.08;
+
+use base 'Exporter';
 
-our @ISA = ('Exporter');
 our @EXPORT = (
         'startMessage',
         'endMessage',
@@ -26,10 +29,11 @@ our @EXPORT = (
         'createMessage',
         'createMetadata',
         'createData',
+        'getErrorResponseMessage',
         );
 
 
-sub startMessage($$$$$$) {
+sub startMessage {
     my ($output, $id, $messageIdRef, $type, $content, $namespaces) = @_;  
     my $logger = get_logger("perfSONAR_PS::Messages");
 
@@ -41,17 +45,17 @@ sub startMessage($$$$$$) {
     return $output->startElement(prefix => "nmwg", tag => "message", namespace => "http://ggf.org/ns/nmwg/base/2.0/", attributes => \%attrs, extra_namespaces => $namespaces, content => $content);
 }
 
-sub endMessage($) {
+sub endMessage {
     my ($output) = @_;
 
     return $output->endElement("message");
 }
 
-sub startMetadata($$$$) {
+sub startMetadata {
     my ($output, $id, $metadataIdRef, $namespaces) = @_;  
     my $logger = get_logger("perfSONAR_PS::Messages");
 
-    if (!defined $id or $id eq "") {
+    if (not defined $id or $id eq "") {
         $logger->error("Missing argument(s).");
         return -1;
     }
@@ -63,18 +67,18 @@ sub startMetadata($$$$) {
     return $output->startElement(prefix => "nmwg", tag => "metadata", namespace => "http://ggf.org/ns/nmwg/base/2.0/", attributes => \%attrs, extra_namespaces => $namespaces);
 }
 
-sub endMetadata($) {
+sub endMetadata {
     my ($output) = @_;  
     my $logger = get_logger("perfSONAR_PS::Messages");
 
     return $output->endElement("metadata");
 }
 
-sub startData($$$$) {
+sub startData {
     my ($output, $id, $metadataIdRef, $namespaces) = @_;  
     my $logger = get_logger("perfSONAR_PS::Messages");
 
-    if (!defined $id or $id eq "" or !defined $metadataIdRef or $metadataIdRef eq "") {
+    if (not defined $id or $id eq "" or not defined $metadataIdRef or $metadataIdRef eq "") {
         $logger->debug("createData failed: \"$id\" \"$metadataIdRef\"");
         return -1;
     }
@@ -82,31 +86,38 @@ sub startData($$$$) {
     return $output->startElement(prefix => "nmwg", tag => "data", namespace => "http://ggf.org/ns/nmwg/base/2.0/", attributes => { id=>$id, metadataIdRef=>$metadataIdRef }, extra_namespaces => $namespaces);
 }
 
-sub endData($) {
+sub endData {
     my ($output) = @_;  
     my $logger = get_logger("perfSONAR_PS::Messages");
 
     return $output->endElement("data");
 }
 
-sub startParameters($$) {
+sub startParameters {
     my ($output, $id) = @_;
 
     return $output->startElement(prefix => "nmwg", tag => "parameters", namespace => "http://ggf.org/ns/nmwg/base/2.0/", attributes => { id=>$id });
 }
 
-sub endParameters($) {
+sub endParameters {
     my ($output) = @_;
 
     return $output->endElement("parameters");
 }
 
 # XXX this should probably ensure that the parameters are being created inside a parameters block
-sub addParameter($$$) {
-    my ($output, $name, $value) = @_;
+sub addParameter {
+    my ($output, $name, $value, $args) = @_;
     my $logger = get_logger("perfSONAR_PS::Messages");
-
-    return $output->createElement(prefix => "nmwg", tag => "parameter", namespace => "http://ggf.org/ns/nmwg/base/2.0/", attributes => {name=>$name}, content => $value);
+    
+    # XXX jason 3/6/08 - Fix the parameters hack after conversion to new argument types
+    my %attrs = ();
+    if(defined $args) {
+      %attrs = %{$args};
+    }
+    $attrs{"name"} = $name;
+    
+    return $output->createElement(prefix => "nmwg", tag => "parameter", namespace => "http://ggf.org/ns/nmwg/base/2.0/", attributes => \%attrs, content => $value);
 }
 
 sub getResultCodeMessage {
@@ -129,11 +140,11 @@ sub getResultCodeMessage {
     return 0;
 }
 
-sub getResultCodeMetadata($$$$) {
+sub getResultCodeMetadata {
     my ($output, $id, $metadataIdRef, $event) = @_; 
     my $logger = get_logger("perfSONAR_PS::Messages");
 
-    if (!defined $id or $id eq "" or !defined $event or $event eq "") {
+    if (not defined $id or $id eq "" or not defined $event or $event eq "") {
         $logger->error("Missing argument(s).");
         return -1;
     }
@@ -153,11 +164,11 @@ sub getResultCodeMetadata($$$$) {
 }
 
 # Changes: adds an 'escape_content' parameter at the end
-sub getResultCodeData($$$$$) {
+sub getResultCodeData {
     my ($output, $id, $metadataIdRef, $description, $escape_content) = @_;  
     my $logger = get_logger("perfSONAR_PS::Messages");
 
-    if (!defined $id or $id eq "" or !defined $metadataIdRef or $metadataIdRef eq "" or !defined $description or $description eq "") {
+    if (not defined $id or $id eq "" or not defined $metadataIdRef or $metadataIdRef eq "" or not defined $description or $description eq "") {
         return -1;
     }
 
@@ -173,7 +184,7 @@ sub getResultCodeData($$$$$) {
     return 0;
 }
 
-sub statusReport($$$$$$) {
+sub statusReport {
     my ($output, $mdId, $mdIdRef, $dId, $eventType, $msg) = @_;
     my $logger = get_logger("perfSONAR_PS::Messages");
 
@@ -184,7 +195,7 @@ sub statusReport($$$$$$) {
     return getResultCodeData($output, $dId, $mdId, $msg, 1); 
 }
 
-sub createMessage($$$$$$) {
+sub createMessage {
     my ($output, $id, $messageIdRef, $type, $content, $namespaces) = @_;  
     my $logger = get_logger("perfSONAR_PS::Messages");
 
@@ -195,11 +206,11 @@ sub createMessage($$$$$$) {
     return endMessage($output);
 }
 
-sub createMetadata($$$$$) {
+sub createMetadata {
     my ($output, $id, $metadataIdRef, $content, $namespaces) = @_;  
     my $logger = get_logger("perfSONAR_PS::Messages");
 
-    if (!defined $id or $id eq "") {
+    if (not defined $id or $id eq "") {
         $logger->error("Missing argument(s).");
         return -1;
     }
@@ -213,11 +224,11 @@ sub createMetadata($$$$$) {
     return $output->endElement("metadata");
 }
 
-sub createData($$$$$) {
+sub createData {
     my ($output, $id, $metadataIdRef, $content, $namespaces) = @_;  
     my $logger = get_logger("perfSONAR_PS::Messages");
 
-    if (!defined $id or $id eq "" or !defined $metadataIdRef or $metadataIdRef eq "") {
+    if (not defined $id or $id eq "" or not defined $metadataIdRef or $metadataIdRef eq "") {
         $logger->debug("createData failed: \"$id\" \"$metadataIdRef\"");
         return -1;
     }
@@ -228,6 +239,43 @@ sub createData($$$$$) {
     return 0;
 }
 
+sub getErrorResponseMessage {
+	my $args = validate(@_, 
+			{
+				output => { optional => 1 },
+				id => { type => SCALAR | UNDEF, optional => 1 },
+				messageIdRef => { type => SCALAR | UNDEF, optional => 1 },
+				metadataIdRef => { type => SCALAR | UNDEF, optional => 1 },
+				eventType => { type => SCALAR },
+				description => { type => SCALAR },
+			});
+
+    my $logger = get_logger("perfSONAR_PS::Messages");
+
+    my $output = $args->{output};
+    my $id = $args->{id};
+    my $messageIdRef = $args->{messageIdRef};
+    my $metadataIdRef = $args->{messageIdRef};
+    my $eventType = $args->{eventType};
+    my $description = $args->{description};
+
+    if (not defined $args->{id}) {
+        $id = "message.".genuid();
+    }
+
+    if (not defined $args->{output}) {
+        $output = new perfSONAR_PS::XML::Document_string();
+    }
+
+    my $n = getResultCodeMessage($output, $id, $messageIdRef, $metadataIdRef, "ErrorResponse", $eventType, $description, undef, 0);
+
+    if (not defined $args->{output}) {
+        return $output->getValue;
+    } else {
+        return 0;
+    }
+}
+
 1;
 
 
@@ -339,7 +387,7 @@ with this software.  If not, see <http://www.internet2.edu/membership/ip.html>
 
 =head1 COPYRIGHT
  
-Copyright (c) 2004-2007, Internet2 and the University of Delaware
+Copyright (c) 2004-2008, Internet2 and the University of Delaware
 
 All rights reserved.
 
