@@ -1,16 +1,29 @@
 package perfSONAR_PS::XML::Document_string;
 
-our $VERSION = 0.06;
+=head1 NAME
 
-use fields 'OPEN_TAGS', 'DEFINED_PREFIXES', 'STRING';
+perfSONAR_PS::XML::Document_string - This module is used to provide a more
+abstract method for constructing XML documents that can be implemented using
+string construction, outputting to a file or even DOM construction without
+tying the code creating the XML to any particular construction method..
+
+=cut
 
 use strict;
+use warnings;
 use Log::Log4perl qw(get_logger :nowarn);
 use Params::Validate qw(:all);
 
+our $VERSION = 0.08;
+
+use fields 'OPEN_TAGS', 'DEFINED_PREFIXES', 'STRING';
+
 my $pretty_print = 0;
 
-sub new($) {
+=head2 new ($package)
+    Allocate a new XML Document
+=cut
+sub new {
 	my ($package) = @_;
 
 	my $self = fields::new($package);
@@ -22,7 +35,10 @@ sub new($) {
 	return $self;
 }
 
-sub getNormalizedURI($) {
+=head2 getNormalizedURI ($uri)
+    This function ensures the URI has no whitespace and ends in a '/'.
+=cut
+sub getNormalizedURI {
 	my ($uri) = @_;
 
 	# trim whitespace
@@ -36,8 +52,23 @@ sub getNormalizedURI($) {
 	return $uri;
 }
 
+=head2 startElement ($self, { prefix, namespace, tag, attributes, extra_namespaces, content })
+    This function starts a new element 'tag' with the prefix 'prefix' and
+    namespace 'namespace'. Those elements are the only ones that are required.
+    The attributes parameter can point at a hash whose keys will become
+    attributes of the element with the value of the attribute being the value
+    corresponding to that key in the hash. The extra_namespaces parameter can
+    be specified to add namespace declarations to this element. The keys of the
+    hash will be the new prefixes and the values those keys point to will be
+    the new namespace URIs. The content parameter can be specified to give the
+    content of the element in which case more elements can still be added, but
+    initally the content will be added. Once started, the element must be
+    closed before the document can be retrieved. This function returns -1 if an
+    error occurs and 0 if the element was successfully created.
+=cut
 sub startElement {
-	my $self = shift;
+	#my ($self, @params) = shift;
+    my $self = shift;
 	my $args = validate(@_, 
 			{
 				prefix => { type => SCALAR, regex => qr/^[a-z0-9]/ },
@@ -65,15 +96,15 @@ sub startElement {
 	$namespaces{$prefix} = $namespace;
 
 	if (defined $extra_namespaces and $extra_namespaces ne "") {
-		foreach $prefix (keys %{ $extra_namespaces }) {
-			my $new_namespace = getNormalizedURI($extra_namespaces->{$prefix});
+		foreach my $curr_prefix (keys %{ $extra_namespaces }) {
+			my $new_namespace = getNormalizedURI($extra_namespaces->{$curr_prefix});
 
-			if (defined $namespaces{$prefix} and $namespaces{$prefix} ne $new_namespace) {
-				$logger->error("Tried to redefine prefix $prefix from ".$namespaces{$prefix}." to ".$new_namespace);
+			if (defined $namespaces{$curr_prefix} and $namespaces{$curr_prefix} ne $new_namespace) {
+				$logger->error("Tried to redefine prefix $curr_prefix from ".$namespaces{$curr_prefix}." to ".$new_namespace);
 				return -1;
 			}
 
-			$namespaces{$prefix} = $new_namespace;
+			$namespaces{$curr_prefix} = $new_namespace;
 		}
 	}
 
@@ -94,7 +125,7 @@ sub startElement {
 	foreach my $prefix (keys %namespaces) {
 		my $require_defintion = 0;
 
-		if (!defined $self->{DEFINED_PREFIXES}->{$prefix}) {
+		if (not defined $self->{DEFINED_PREFIXES}->{$prefix}) {
 			# it's the first time we've seen a prefix like this
 			$self->{DEFINED_PREFIXES}->{$prefix} = ();
 			push @{ $self->{DEFINED_PREFIXES}->{$prefix} }, $namespaces{$prefix};
@@ -103,7 +134,7 @@ sub startElement {
 			my @namespaces = @{ $self->{DEFINED_PREFIXES}->{$prefix} };
 
 			# if it's a new namespace for an existing prefix, write the definition (though we should probably complain)
-			if ($#namespaces == -1 or $namespaces[$#namespaces] ne $namespace) {
+			if ($#namespaces == -1 or $namespaces[-1] ne $namespace) {
 				push @{ $self->{DEFINED_PREFIXES}->{$prefix} }, $namespaces{$prefix};
 
 				$require_defintion = 1;
@@ -139,6 +170,11 @@ sub startElement {
 	return 0;
 }
 
+=head2 createElement ($self, { prefix, namespace, tag, attributes, extra_namespaces, content })
+    This function has identical parameters to the startElement function.
+    However, it closes the element immediately. This function returns -1 if an
+    error occurs and 0 if the element was successfully created.
+=cut
 sub createElement {
 	my $self = shift;
 	my $args = validate(@_, 
@@ -166,15 +202,15 @@ sub createElement {
 	$namespaces{$prefix} = $namespace;
 
 	if (defined $extra_namespaces and $extra_namespaces ne "") {
-		foreach $prefix (keys %{ $extra_namespaces }) {
-			my $new_namespace = getNormalizedURI($extra_namespaces->{$prefix});
+		foreach my $curr_prefix (keys %{ $extra_namespaces }) {
+			my $new_namespace = getNormalizedURI($extra_namespaces->{$curr_prefix});
 
-			if (defined $namespaces{$prefix} and $namespaces{$prefix} ne $new_namespace) {
-				$logger->error("Tried to redefine prefix $prefix from ".$namespaces{$prefix}." to ".$new_namespace);
+			if (defined $namespaces{$curr_prefix} and $namespaces{$curr_prefix} ne $new_namespace) {
+				$logger->error("Tried to redefine prefix $curr_prefix from ".$namespaces{$curr_prefix}." to ".$new_namespace);
 				return -1;
 			}
 
-			$namespaces{$prefix} = $new_namespace;
+			$namespaces{$curr_prefix} = $new_namespace;
 		}
 	}
 
@@ -189,7 +225,7 @@ sub createElement {
 	foreach my $prefix (keys %namespaces) {
 		my $require_defintion = 0;
 
-		if (!defined $self->{DEFINED_PREFIXES}->{$prefix}) {
+		if (not defined $self->{DEFINED_PREFIXES}->{$prefix}) {
 			# it's the first time we've seen a prefix like this
 			$self->{DEFINED_PREFIXES}->{$prefix} = ();
 			$require_defintion = 1;
@@ -197,7 +233,7 @@ sub createElement {
 			my @namespaces = @{ $self->{DEFINED_PREFIXES}->{$prefix} };
 
 			# if it's a new namespace for an existing prefix, write the definition (though we should probably complain)
-			if ($#namespaces == -1 or $namespaces[$#namespaces] ne $namespace) {
+			if ($#namespaces == -1 or $namespaces[-1] ne $namespace) {
 				$require_defintion = 1;
 			}
 		}
@@ -213,7 +249,7 @@ sub createElement {
 		}
 	}
 
-	if (!defined $content or $content eq "") {
+	if (not defined $content or $content eq "") {
 		$self->{STRING} .= " />";
 	} else {
 		$self->{STRING} .= ">";
@@ -243,7 +279,12 @@ sub createElement {
 	return 0;
 }
 
-sub endElement($$) {
+=head2 endElement ($self, $tag)
+    This function is used to end the most recently opened element. The tag
+    being closed is specified to sanity check the output. If the element is
+    properly closed, 0 is returned. -1 otherwise.
+=cut
+sub endElement {
 	my ($self, $tag) = @_;
 	my $logger = get_logger("perfSONAR_PS::XML::Document_string");
 
@@ -254,12 +295,12 @@ sub endElement($$) {
     if ($#tags == -1) {
         $logger->error("Tried to close tag $tag but no current open tags");
 		return -1;
-	} elsif ($tags[$#tags]->{"tag"} ne $tag) {
-        $logger->error("Tried to close tag $tag, but current open tag is \"".$tags[$#tags]->{"tag"}."\n");
+	} elsif ($tags[-1]->{"tag"} ne $tag) {
+        $logger->error("Tried to close tag $tag, but current open tag is \"".$tags[-1]->{"tag"}."\n");
 		return -1;
 	}
 
-	foreach my $prefix (@{ $tags[$#tags]->{"defined_prefixes"} }) {
+	foreach my $prefix (@{ $tags[-1]->{"defined_prefixes"} }) {
 		pop @{ $self->{DEFINED_PREFIXES}->{$prefix} };
 	}
 
@@ -271,7 +312,7 @@ sub endElement($$) {
 		}
 	}
 
-	$self->{STRING} .= "</".$tags[$#tags]->{"prefix"}.":".$tag.">";
+	$self->{STRING} .= "</".$tags[-1]->{"prefix"}.":".$tag.">";
 
 	if ($pretty_print) {
 		$self->{STRING} .= "\n";
@@ -280,16 +321,25 @@ sub endElement($$) {
 	return 0;
 }
 
-sub addExistingXMLElement($$) {
+=head2 addExistingXMLElement ($self, $element)
+    This function adds a LibXML element to the current document.
+=cut
+sub addExistingXMLElement {
 	my ($self, $element) = @_;
 	my $logger = get_logger("perfSONAR_PS::XML::Document_string");
 
-	$self->{STRING} .= $element->toString();
+    my $elm = $element->cloneNode(1);
+    $elm->unbindNode();
+
+	$self->{STRING} .= $elm->toString();
 
 	return 0;
 }
 
-sub addOpaque($$) {
+=head2 addOpaque ($self, $element)
+    This function adds arbitrary data to the current document.
+=cut
+sub addOpaque {
 	my ($self, $data) = @_;
 	my $logger = get_logger("perfSONAR_PS::XML::Document_string");
 
@@ -298,7 +348,11 @@ sub addOpaque($$) {
 	return 0;
 }
 
-sub getValue($) {
+=head2 getValue ($self)
+    This function returns the current state of the document. It will warn if
+    there are open tags still.
+=cut
+sub getValue {
 	my ($self) = @_;
 	my $logger = get_logger("perfSONAR_PS::XML::Document_string");
 
@@ -309,7 +363,7 @@ sub getValue($) {
 			my $msg = "Open tags still exist: ";
 
 			for(my $x = $#open_tags; $x >= 0; $x--) {
-				$msg .= " -> ".$open_tags[$x];
+				$msg .= " -> ".$open_tags[$x]->{tag};
 			}
 
 			$logger->warn($msg);
@@ -323,4 +377,44 @@ sub getValue($) {
 
 1;
 
+__END__
+
+=head1 SEE ALSO
+
+L<Log::Log4perl>, L<Params::Validate>
+
+To join the 'perfSONAR-PS' mailing list, please visit:
+
+  https://mail.internet2.edu/wws/info/i2-perfsonar
+
+The perfSONAR-PS subversion repository is located at:
+
+  https://svn.internet2.edu/svn/perfSONAR-PS
+
+Questions and comments can be directed to the author, or the mailing list.
+Bugs, feature requests, and improvements can be directed here:
+
+  https://bugs.internet2.edu/jira/browse/PSPS
+
+=head1 VERSION
+
+$Id: perfSONARBOUY.pm 1059 2008-03-07 02:30:34Z zurawski $
+
+=head1 AUTHOR
+
+Aaron Brown, aaron@internet2.edu
+
+=head1 LICENSE
+
+You should have received a copy of the Internet2 Intellectual Property Framework
+along with this software.  If not, see
+<http://www.internet2.edu/membership/ip.html>
+
+=head1 COPYRIGHT
+
+Copyright (c) 2004-2008, Internet2 and the University of Delaware
+
+All rights reserved.
+
+=cut
 # vim: expandtab shiftwidth=4 tabstop=4
